#pragma once
#include "../GPU/gpu_types.hpp"
#include "../../math.h"

namespace JabyEngine {
    namespace GTE {
        namespace internal {
            template<typename T>
            struct VECTOR {
                T x;
                T y;
                T z;
                T pad;

                static constexpr VECTOR create() {
                    return VECTOR::create(0, 0, 0);
                }

                static constexpr VECTOR create(T x, T y, T z) {
                    return VECTOR{.x = x, .y = y, .z = z, .pad = 0};
                }

                template<typename S>
                static constexpr VECTOR from(const GPU::Position<S>& pos) {
                    return VECTOR::create(static_cast<T>(pos.x), static_cast<T>(pos.y), 0);
                }

                template<typename S>
                constexpr S to() const {
                    return S::create(static_cast<S::PrimitiveType>(this->x), static_cast<S::PrimitiveType>(this->y));
                }
            };
        }

        // TODO: Split MATRIX into ROTMATRIX and TRANSFERVECTOR or something like that. But first get it running
        struct MATRIX {
            int16_t rot[3][3]; // Rotation matrix
            int32_t trans[3];  // Translation vector

            static constexpr MATRIX identity() {
                return MATRIX{
                    .rot = {
                        {4096, 0, 0},
                        {0, 4096, 0},
                        {0, 0, 4096}
                    },
                    .trans = {0, 0, 0}
                };
            }

            static MATRIX rotation(int32_t x, int32_t y, int32_t z);
        };

        using VECTOR  = internal::VECTOR<int32_t>;
        using SVECTOR = internal::VECTOR<int16_t>;

        static constexpr auto one_degree       = FULL_CIRCLE/360;
        static constexpr auto one_tenth_degree = FULL_CIRCLE/3600;

        static constexpr int16_t in_degree(int32_t degree) {
            return degree*one_degree;
        }

        static constexpr int16_t in_degree10(int32_t degree) {
            return degree*one_tenth_degree;
        }
    }

    static constexpr unsigned long long operator""_DEG(unsigned long long degree) {
        return GTE::in_degree(degree);
    }

    static constexpr unsigned long long operator""_DEG10(unsigned long long degree) {
        return GTE::in_degree10(degree);
    }
}